/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.extension;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.jar.JarFile;
import java.util.jar.Manifest;
import org.apache.avalon.excalibur.io.FileUtil;
import org.apache.avalon.excalibur.util.DeweyDecimal;

/**
 * <p>Interface used to contain "Optional Packages" (formerly known as 
 * "Standard Extensions"). It is assumed that each "Optional Package" is 
 * represented by a single file on the file system. This Repository manages
 * a single sub-tree on file system.</p>
 *
 * <p>Each package directory contains a data file named 
 * <code>packages.cache</code>. This file caches relevent meta-information
 * about "Optional Packages" and maps that to a specific file.</p>
 *
 * <p>The filename for each "Optional Package" is derived from a simple 
 * algorithm. The first <code>Extension</code> listed in "Optional Packages" manifest
 * is used to name package. The <code>Implementation-Vendor-ID</code> and 
 * <code>Extension-Name</code> are processed so that each string only contains 
 * valid characters. The set of valid characters include all ascii
 * alphanumeric characters and '.'. Invalid characters are replaced with the
 * '_' character.</p>
 *
 * <p>The filename is then constructed using these components and the 
 * <code>Implementation-Version</code> using the following pattern.</p>
 *
 * <pre>
 * <code>&lt;Implementation-Vendor-ID&gt;/&lt;Extension-Name&gt;-&lt;Implementation-Version&gt;.jar</code>
 * </pre>
 *
 * @author <a href="mailto:donaldp@apache.org">Peter Donald</a>
 * @version $Revision: 1.1 $ $Date: 2001/09/03 03:21:02 $
 * @see OptionalPackage
 * @see PackageRepository
 */
public class DefaultPackageRepository
    implements PackageRepository
{
    ///Map between files and <code>OptionalPackage</code> objects.
    private final HashMap    m_packages = new HashMap();

    ///Base directory of repository
    private final File       m_baseDirectory;

    /**
     * Construct a package repository with specified base directory.
     *
     * @param baseDirectory the base directory for repository
     */
    public DefaultPackageRepository( final File baseDirectory )
    {
        m_baseDirectory = baseDirectory;

        if( null == m_baseDirectory )
        {
            throw new NullPointerException( "baseDirectory property is null" );
        }

        if( !m_baseDirectory.isDirectory() )
        {
            throw new IllegalArgumentException( "baseDirectory must be a directory" );
        }
    }

    /**
     * Return the <code>OptionalPackage</code> that provides specified
     * <code>Extension</code>. It is expected that this <code>Extension</code>
     * object will be one retrieved via getLocalExtension() method. If the
     * specified <code>Extension</code> is not local then <code>null</code>
     * is returned.
     *
     * @param extension Description of the optional package
     * @see #isLocal()
     */
    public OptionalPackage getOptionalPackage( final Extension extension )
    {
        final OptionalPackage[] packages = getOptionalPackages( extension );

        //If theres a candidate package then return it
        //else return null
        if( null != packages || 0 != packages.length )
        {
            return packages[ 0 ];
        }
        else
        {
            return null;
        }
    }

    /**
     * Return all the <code>OptionalPackage</code>s that satisfy specified
     * <code>Extension</code>. It is expected that this <code>Extension</code>
     * object will be one retrieved via getLocalExtension() method. If the
     * specified <code>Extension</code> is not local then <code>null</code>
     * is returned.
     *
     * @param extension Description of the optional package
     * @see #isLocal()
     */
    public synchronized OptionalPackage[] getOptionalPackages( final Extension extension )
    {
        final ArrayList results = new ArrayList();

        //TODO: Check cache consistency and reload if necessary..
        final ArrayList candidates = (ArrayList)m_packages.get( extension.getExtensionName() );
        if( null != candidates )
        {
            final int size = candidates.size();
            for( int i = 0; i < size; i++ )
            {
                final OptionalPackage optionalPackage = (OptionalPackage)candidates.get( i );
                final Extension[] extensions = optionalPackage.getAvailableExtensions();

                for( int j = 0; j < extensions.length; j++ )
                {
                    if( extensions[ j ].isCompatibleWith( extension ) )
                    {
                        results.add( optionalPackage );
                    }
                }
            }
        }

        //TODO: Sort packages so that most relevent is first
        //ie Sort on spec version first and then on Imp version

        return (OptionalPackage[])results.toArray( new OptionalPackage[ 0 ] );
    }

    /**
     * Install the specified <code>Extension</code> (which represents
     * an optional package required by another package). The mechanism
     * and location of installation is not specified.
     *
     * <p>It is an error for an extension with the same
     * <code>Implementation-Version</code>, <code>Implementation-Vendor-ID</code>
     * and <code>Extension-Name</code> to already exist on local system.</p>
     *
     * <p>It is expected that after successful completion of this
     * method that an OptionalPackage will be accessible via getOptionalPackage().</p>
     *
     * @param file the file containing "Optional Package".
     * @exception IOException if an error occurs
     */
    public void install( final File inputFile )
        throws IOException, SecurityException
    {
        final OptionalPackage preInstallPackage = getOptionalPackage( inputFile );
        final Extension[] extensions = preInstallPackage.getAvailableExtensions();

        if( 0 == extensions.length )
        {
            throw new IOException( "File contains no extensions and thus " +
                                   "not a valid Optional Package" );
        }

        final String filename = getFilename( extensions[ 0 ] );

        //Create file beneath writeable repository here
        final File file = (new File( m_baseDirectory, filename )).getCanonicalFile();
        FileUtil.copyFile( inputFile, file );

        final OptionalPackage optionalPackage =
            new OptionalPackage( file,
                                 preInstallPackage.getAvailableExtensions(),
                                 preInstallPackage.getRequiredExtensions() );

        cacheOptionalPackage( optionalPackage );

        //Update cache on filesystem here...
    }

    /**
     * Add OptionalPackage to internal cache of Optional Packages.
     * Note that this method is only protected so that unit tests can sub-class
     * and add entries to PackageRepository by calling this method.
     *
     * @param optionalPackage the OptionalPackage to be added to repository
     */
    protected final void cacheOptionalPackage( final OptionalPackage optionalPackage )
    {
        final Extension extension = optionalPackage.getAvailableExtensions()[ 0 ];
        ArrayList candidates = (ArrayList)m_packages.get( extension.getExtensionName() );
        if( null == candidates )
        {
            candidates = new ArrayList();
            m_packages.put( extension.getExtensionName(), candidates );
        }

        //TODO: Add this in descending order so that don't have to sort in
        //getOptionalPackages ????
        //yes, sort by Spec Version then vendor, Imp Version
        candidates.add( optionalPackage );
    }

    /**
     * Construct an OptionalPackage out of the specified jar archive.
     *
     * @param archive the file object for Jar archive
     * @return the OptionalPackage constructed
     * @exception IOException if an error occurs
     */
    private OptionalPackage getOptionalPackage( final File archive )
        throws IOException
    {
        final File file = archive.getCanonicalFile();
        final JarFile jarFile = new JarFile( file );
        final Manifest manifest = jarFile.getManifest();

        try
        {
            if( null == manifest ) return null;
            final Extension[] available = ExtensionUtil.getAvailableExtensions( manifest );
            final Extension[] required = ExtensionUtil.getRequiredExtensions( manifest );

            return new OptionalPackage( file, available, required );
        }
        finally
        {
            jarFile.close();
        }
    }

    /**
     * Generate the filename for specified <code>Extension</code>.
     * This extension must specify <code>Implementation-Vendor-Id</code>
     * and <code>Implementation-Version</code>.
     *
     * @param extension the extension
     * @return the generated filename
     */
    private String getFilename( final Extension extension )
    {
        if( null == extension.getImplementationVendorId() )
        {
            throw new NullPointerException( "ImplementationVendorId is null" );
        }

        if( null == extension.getImplementationVersion() )
        {
            throw new NullPointerException( "ImplementationVersion is null" );
        }

        final String name = getCleanName( extension.getExtensionName() );
        final String implVendorId = getCleanName( extension.getImplementationVendorId() );
        final DeweyDecimal implVersion = extension.getImplementationVersion();

        return implVendorId + File.separator + name + "-" + implVersion + ".jar";
    }

    /**
     * Clean name so that invalid characters are replaced with '_'.
     * The set of valid characters consists of ascii alphanumeric
     * characters and '.'.
     *
     * @param name the input name
     * @return the cleaned name
     */
    private String getCleanName( final String name )
    {
        final int size = name.length();
        final StringBuffer sb = new StringBuffer( size );

        for( int i = 0; i < size; i++ )
        {
            final char ch = name.charAt( i );

            if( Character.isLetterOrDigit( ch ) || '.' == ch )
            {
                sb.append( ch );
            }
            else
            {
                sb.append( '_' );
            }
        }

        return sb.toString();
    }
}
