/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.extension;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.jar.Attributes.Name;
import java.util.jar.Attributes;
import java.util.jar.Manifest;
import org.apache.avalon.excalibur.util.DeweyDecimal;
import org.apache.avalon.excalibur.util.StringUtil;

/**
 * <p>Utility class that represents either an available "Optional Package"
 * (formerly known as "Standard Extension") as described in the manifest
 * of a JAR file, or the requirement for such an optional package.</p>
 *
 * <p>For more information about optional packages, see the document
 * <em>Optional Package Versioning</em> in the documentation bundle for your
 * Java2 Standard Edition package, in file
 * <code>guide/extensions/versioning.html</code>.</p>
 *
 * @author <a href="mailto:craigmcc@apache.org">Craig R. McClanahan</a>
 * @author <a href="mailto:donaldp@apache.org">Peter Donald</a>
 * @version $Revision: 1.1 $ $Date: 2001/09/03 03:21:02 $
 */
public final class Extension
{
    /**
     * The name of the optional package being made available, or required.
     */
    private String        m_extensionName;

    /**
     * The version number (dotted decimal notation) of the specification
     * to which this optional package conforms.
     */
    private DeweyDecimal  m_specificationVersion;

    /**
     * The name of the company or organization that originated the
     * specification to which this optional package conforms.
     */
    private String        m_specificationVendor;

    /**
     * The URL from which the most recent version of this optional package
     * can be obtained if it is not already installed.
     */
    private String        m_implementationURL;

    /**
     * The name of the company or organization that produced this
     * implementation of this optional package.
     */
    private String        m_implementationVendor;

    /**
     * The unique identifier of the company that produced the optional
     * package contained in this JAR file.
     */
    private String        m_implementationVendorId;

    /**
     * The version number (dotted decimal notation) for this implementation
     * of the optional package.
     */
    private DeweyDecimal  m_implementationVersion;

    /**
     * The constructor to create Extension object.
     * Note that every component is allowed to be specified
     * but only the extensionName is mandatory.
     *
     * @param extensionName the name of extension.
     * @param specificationVersion the specification Version of extension.
     * @param specificationVendor the specification Vendor of extension.
     * @param implementationVersion the implementation Version of extension.
     * @param implementationVendor the implementation Vendor of extension.
     * @param implementationVendorId the implementation VendorId of extension.
     * @param implementationURL the implementation URL of extension.
     */
    public Extension( final String extensionName,
                      final String specificationVersion,
                      final String specificationVendor,
                      final String implementationVersion,
                      final String implementationVendor,
                      final String implementationVendorId,
                      final String implementationURL )
    {
        m_extensionName = extensionName;
        m_specificationVendor = specificationVendor;

        if( null != specificationVersion )
        {
            m_specificationVersion = new DeweyDecimal( specificationVersion );
        }

        m_implementationURL = implementationURL;
        m_implementationVendor = implementationVendor;
        m_implementationVendorId = implementationVendorId;

        if( null != implementationVersion )
        {
            m_implementationVersion = new DeweyDecimal( implementationVersion );
        }

        if( null == m_extensionName )
        {
            throw new NullPointerException( "extensionName property is null" );
        }
    }

    public String getExtensionName()
    {
        return m_extensionName;
    }

    public String getSpecificationVendor()
    {
        return m_specificationVendor;
    }

    public DeweyDecimal getSpecificationVersion()
    {
        return m_specificationVersion;
    }

    public String getImplementationURL()
    {
        return m_implementationURL;
    }

    public String getImplementationVendor()
    {
        return m_implementationVendor;
    }

    public String getImplementationVendorId()
    {
        return m_implementationVendorId;
    }

    public DeweyDecimal getImplementationVersion()
    {
        return m_implementationVersion;
    }

    /**
     * Return <code>true</code> if the specified <code>Extension</code>
     * (which represents an optional package required by an application)
     * is satisfied by this <code>Extension</code> (which represents an
     * optional package that is already installed.  Otherwise, return
     * <code>false</code>.
     *
     * @param required Description of the required optional package
     */
    public boolean isCompatibleWith( final Extension required )
    {
        // Extension Name must match
        if( false == m_extensionName.equals( required.getExtensionName() ) )
        {
            return false;
        }

        // Available specification version must be >= required
        final DeweyDecimal specificationVersion = required.getSpecificationVersion();
        if( null != specificationVersion )
        {
            if( null == m_specificationVersion ||
                false == isCompatible( m_specificationVersion, specificationVersion ) )
            {
                return false;
            }
        }

        // Implementation Vendor ID must match
        final String implementationVendorId = required.getImplementationVendorId();
        if( null != implementationVendorId )
        {
            if( null == m_implementationVendorId ||
                false == m_implementationVendorId.equals( implementationVendorId ) )
            {
                return false;
            }
        }

        // Implementation version must be >= required
        final DeweyDecimal implementationVersion = required.getImplementationVersion();
        if( null != implementationVersion )
        {
            if( null == m_implementationVersion ||
                false == isCompatible( m_implementationVersion, implementationVersion ) )
            {
                return false;
            }
        }

        // This available optional package satisfies the requirements
        return true;
    }

    /**
     * Return a String representation of this object.
     */
    public String toString()
    {
        final StringBuffer sb = new StringBuffer( "Extension[" );
        sb.append( m_extensionName );

        if( null != m_implementationURL )
        {
            sb.append( ", implementationURL=" );
            sb.append( m_implementationURL );
        }

        if( null != m_implementationVendor )
        {
            sb.append( ", implementationVendor=" );
            sb.append( m_implementationVendor );
        }

        if( null != m_implementationVendorId )
        {
            sb.append( ", implementationVendorId=" );
            sb.append( m_implementationVendorId );
        }

        if( null != m_implementationVersion )
        {
            sb.append( ", implementationVersion=" );
            sb.append( m_implementationVersion );
        }

        if( null != m_specificationVendor )
        {
            sb.append( ", specificationVendor=" );
            sb.append( m_specificationVendor );
        }

        if( null != m_specificationVersion )
        {
            sb.append( ", specificationVersion=" );
            sb.append( m_specificationVersion );
        }

        sb.append( "]" );

        return sb.toString();
    }

    /**
     * Return <code>true</code> if the first version number is greater than
     * or equal to the second; otherwise return <code>false</code>.
     *
     * @param first First version number (dotted decimal)
     * @param second Second version number (dotted decimal)
     *
     * @exception NumberFormatException on a malformed version number
     */
    private boolean isCompatible( final DeweyDecimal first, final DeweyDecimal second )
    {
        return first.isGreaterThanOrEqual( second );
    }
}
