/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.extension;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.jar.JarFile;
import java.util.jar.Attributes.Name;
import java.util.jar.Attributes;
import java.util.jar.Manifest;
import org.apache.avalon.excalibur.util.StringUtil;

/**
 * Utility class that contains methods to scan a manifest
 * and return an array of either available or required optional packages
 * documented in that manifest.
 * <p>
 * For more information about optional packages, see the document
 * <em>Optional Package Versioning</em> in the documentation bundle for your
 * Java2 Standard Edition package, in file
 * <code>guide/extensions/versioning.html</code>.
 *
 * @author <a href="mailto:craigmcc@apache.org">Craig R. McClanahan</a>
 * @author <a href="mailto:donaldp@apache.org">Peter Donald</a>
 * @version $Revision: 1.2 $ $Date: 2001/09/03 07:58:44 $
 */
public final class ExtensionUtil
{
    /**
     * Private constructor so that class can not be instantiated.
     */
    private ExtensionUtil()
    {
    }

    /**
     * Build a list of dependencies based on specified <code>Extension</code>s.
     * Each specified <code>Extension</code> is expected to be a required extension
     * of another "Optional Package". 
     *
     * <p>If the required <code>Extension</code> can not be found locally then
     * it is placed in list of unsatisfied Extensions. If a candidate <code>Extension</code>
     * is found locally that satisfies specified required <code>Extension</code>
     * then it is added to list of dependencies. scanDependencies() is then recursively
     * called on all of the candidates required extensions.</p>
     *
     * @param manager the PackageManager used to find local Extensions.
     * @param extensions the array of required Extensions.
     * @param dependencies the list of dependencies.
     * @param unsatisfied the list of unsatisfied (ie non-local) dependencies.
     * @see #scanDependencies
     */
    public static void scanDependencies( final PackageManager manager, 
                                         final Extension[] extensions, 
                                         final List dependencies,
                                         final List unsatisfied )
    {
        for( int i = 0; i < extensions.length; i++ )
        {
            scanDependencies( manager, extensions[ i ], dependencies, unsatisfied );
        }
    }

    /**
     * Build a list of dependencies based on specified <code>Extension</code>.
     * The specified <code>Extension</code> is expected to be a required extension
     * of another "Optional Package". 
     *
     * <p>If the required <code>Extension</code> can not be found locally then
     * it is placed in list of unsatisfied Extensions. If a candidate <code>OptionalPackage</code>
     * is found locally that satisfies specified required <code>Extension</code>
     * then it is added to list of dependencies. scanDependencies() is then recursively
     * called on all of the candidates required extensions.</p>
     *
     * @param manager the PackageManager used to find local Extensions.
     * @param extension the required Extension.
     * @param dependencies the list of OptionalPackages required to satisfy extension.
     * @param unsatisfied the list of unsatisfied (ie non-local) dependencies.
     * @see #scanDependencies
     */
    public static void scanDependencies( final PackageManager manager, 
                                         final Extension extension, 
                                         final List dependencies,
                                         final List unsatisfied )
    {
        final int size = dependencies.size();

        //Check to make sure one of the already existing 
        for( int i = 0; i < size; i++ )
        {
            final OptionalPackage optionalPackage = (OptionalPackage)dependencies.get( i );
            if( isCompatible( extension, optionalPackage ) )
            {
                return;
            }
        }

        final OptionalPackage optionalPackage = manager.getOptionalPackage( extension );
        if( null == optionalPackage )
        {
            if( !unsatisfied.contains( extension ) )
            {
                unsatisfied.add( extension );
            }
        }
        else
        {
            if( !dependencies.contains( optionalPackage ) )
            {
                dependencies.add( optionalPackage );
            }

            scanDependencies( manager, optionalPackage.getRequiredExtensions(), dependencies, unsatisfied );
        }
    }

    public static boolean isCompatible( final Extension extension, 
                                        final OptionalPackage optionalPackage ) 
    {
        final Extension[] extensions = optionalPackage.getAvailableExtensions();
        for( int i = 0; i < extensions.length; i++ )
        {
            if( extensions[ i ].isCompatibleWith( extension ) )
            {
                return true;
            }
        }

        return false;
    }
    
    /**
     * Return an array of <code>URL</code> objects that contain 
     * specified <code>Extension</code>s. The <code>Extension</code>s
     * specified must be to local Extensions.
     *
     * @param manager the PackageManager from which to retrieve local file information
     * @param extensions the list of local Extensions
     * @return the local URLs that represent extensions
     * @exception IOException if an error occurs
     * @exception IllegalArgumentException if an extension in array is not a local Extension
     */
    public static URL[] getLocalURLs( final PackageManager manager, 
                                      final Extension[] extensions )
        throws IOException, IllegalArgumentException
    {
        final URL[] results = new URL[ extensions.length ];

        for( int i = 0; i < extensions.length; i++ )
        {
            final OptionalPackage optionalPackage = manager.getOptionalPackage( extensions[ i ] );
            if( null == optionalPackage )
            {
                final String message = 
                    "Extension " + extensions[ i ].getExtensionName() + " is not local";
                throw new IllegalArgumentException( message );
            }

            final File file = optionalPackage.getFile();
            results[ i ] = file.toURL();
        }

        return results;
    }

    /**
     * Return an array of <code>File</code> objects that contain 
     * specified <code>Extension</code>s. The <code>Extension</code>s
     * specified must be to local Extensions.
     *
     * @param manager the PackageManager from which to retrieve local file information
     * @param extensions the list of local Extensions
     * @return the local Files that represent extensions
     * @exception IOException if an error occurs
     * @exception IllegalArgumentException if an extension in array is not a local Extension
     */
    public static File[] getLocalFiles( final PackageManager manager, 
                                        final Extension[] extensions )
        throws IOException, IllegalArgumentException
    {
        final File[] results = new File[ extensions.length ];

        for( int i = 0; i < extensions.length; i++ )
        {
            final OptionalPackage optionalPackage = manager.getOptionalPackage( extensions[ i ] );
            if( null == optionalPackage )
            {
                final String message = 
                    "Extension " + extensions[ i ].getExtensionName() + " is not local";
                throw new IllegalArgumentException( message );
            }
            results[ i ] = optionalPackage.getFile();
        }

        return results;
    }

    /**
     * Return an array of <code>Extension</code> objects representing optional
     * packages that are available in the JAR file associated with the
     * specified <code>Manifest</code>.  If there are no such optional
     * packages, a zero-length array is returned.
     *
     * @param manifest Manifest to be parsed
     */
    public static Extension[] getAvailableExtensions( final Manifest manifest )
    {
        if( null == manifest ) return new Extension[ 0 ];

        final ArrayList results = new ArrayList();

        final Attributes mainAttributes = manifest.getMainAttributes();
        if( null != mainAttributes )
        {
            final Extension extension = getExtension( "", mainAttributes );
            if( null != extension ) results.add( extension );
        }

        final Map entries = manifest.getEntries();
        final Iterator keys = entries.keySet().iterator();
        while( keys.hasNext() )
        {
            final String key = (String)keys.next();
            final Attributes attributes = (Attributes)entries.get( key );
            final Extension extension = getExtension( "", attributes );
            if( null != extension ) results.add( extension );
        }

        return (Extension[])results.toArray( new Extension[ 0 ] );
    }

    /**
     * Return the set of <code>Extension</code> objects representing optional
     * packages that are required by the application contained in the JAR
     * file associated with the specified <code>Manifest</code>.  If there
     * are no such optional packages, a zero-length list is returned.
     *
     * @param manifest Manifest to be parsed
     */
    public static Extension[] getRequiredExtensions( final Manifest manifest )
    {
        final ArrayList results = new ArrayList();
        final Attributes mainAttributes = manifest.getMainAttributes();

        if( null != mainAttributes )
        {
            getRequired( mainAttributes, results );
        }

        final Map entries = manifest.getEntries();
        final Iterator keys = entries.keySet().iterator();
        while( keys.hasNext() )
        {
            final String key = (String)keys.next();
            final Attributes attributes = (Attributes)entries.get( key );
            getRequired( mainAttributes, results );
        }

        return (Extension[])results.toArray( new Extension[ 0 ] );
    }

    /**
     * Add required optional packages defined in the specified attributes entry, if any.
     *
     * @param attributes Attributes to be parsed
     * @param required list to add required optional packages to
     */
    private static void getRequired( final Attributes attributes,
                                     final ArrayList required )
    {
        final String names = attributes.getValue( Name.EXTENSION_LIST );
        if( null == names ) return;

        final String[] extentions = StringUtil.split( names, " " );
        for( int i = 0; i < extentions.length; i++ )
        {
            final String prefix = extentions[ i ] + "-";
            final Extension extension = getExtension( prefix, attributes );

            if( null != extension )
            {
                required.add( extension );
            }
        }
    }

    /**
     * Extract an Extension from Attributes.
     * Prefix indicates the prefix checked for each string.
     * Usually the prefix is <em>"&lt;extension&gt;-"</em> if looking for a
     * <b>Required</b> extension. If you are looking for an <b>Available</b> extension
     * then the prefix is <em>""</em>.
     *
     * @param prefix the prefix for each attribute name
     * @param attributes Attributes to searched
     * @return the new Extension object, or null
     */
    private static Extension getExtension( final String prefix, final Attributes attributes )
    {
        final String name = attributes.getValue( prefix + Name.EXTENSION_NAME );
        if( null == name ) return null;

        final String specVendor = attributes.getValue( prefix + Name.SPECIFICATION_VENDOR );
        final String specVersion = attributes.getValue( prefix + Name.SPECIFICATION_VERSION );

        final String impVersion = attributes.getValue( prefix + Name.IMPLEMENTATION_VERSION );
        final String impVendor = attributes.getValue( prefix + Name.IMPLEMENTATION_VENDOR );
        final String impVendorId = attributes.getValue( prefix + Name.IMPLEMENTATION_VENDOR_ID );
        final String impURL = attributes.getValue( prefix + Name.IMPLEMENTATION_URL );

        return new Extension( name, specVersion, specVendor, impVersion,
                              impVendor, impVendorId, impURL );
    }
}
