/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.extension;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

/**
 * Basic Implementation Of PackageManager Interface used to manage 
 * "Optional Packages" (formerly known as "Standard Extensions"). 
 * The "Optional Packages" are stored on file system in a number of
 * directories.
 *
 * @author <a href="mailto:donaldp@apache.org">Peter Donald</a>
 * @version $Revision: 1.1 $ $Date: 2001/09/03 03:21:02 $
 * @see PackageRepository
 */
public class PackageManager
{
    ///Ordered list of repositories to search in
    private PackageRepository[]   m_repositorys;

    /**
     * Construct a PackageManager for a specific set of repositories.
     *
     * @param repositorys the array of repositories to use in PackageManager
     */
    public PackageManager( final PackageRepository[] repositorys )
    {
        m_repositorys = repositorys;

        if( null == repositorys )
        {
            throw new NullPointerException( "repositorys property is null" );
        }
    }

    /**
     * Return the <code>OptionalPackage</code> that provides specified
     * <code>Extension</code>. If the specified <code>Extension</code> 
     * can not be found then <code>null</code> is returned.
     *
     * @param extension Description of the optional package
     * @see #getOptionalPackages()
     * @see OptionalPackage
     * @see Extension
     */
    public OptionalPackage getOptionalPackage( final Extension extension )
    {
        final OptionalPackage[] packages = getOptionalPackages( extension );

        //If theres a candidate package then return it
        //else return null
        if( null != packages || 0 != packages.length )
        {
            return packages[ 0 ];
        }
        else
        {
            return null;
        }
    }

    /**
     * Return all the <code>OptionalPackage</code>s that satisfy specified
     * <code>Extension</code>. 
     *
     * @param extension Description of the optional package
     * @see #getOptionalPackage()
     * @see OptionalPackage
     */
    public synchronized OptionalPackage[] getOptionalPackages( final Extension extension )
    {
        final ArrayList results = new ArrayList();

        for( int i = 0; i < m_repositorys.length; i++ )
        {
            final PackageRepository repository = m_repositorys[ i ];
            final OptionalPackage[] packages = repository.getOptionalPackages( extension );

            for( int j = 0; j < packages.length; j++ )
            {
                results.add( packages[ j ] );
            }
        }

        return (OptionalPackage[])results.toArray( new OptionalPackage[ 0 ] );
    }
}
