/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.extension;

import java.io.File;
import java.io.IOException;

/**
 * <p>Interface used to manage a single collection of "Optional Packages" 
 * (formerly known as "Standard Extensions"). It is assumed that each 
 * "Optional Package" is represented by a single file on the file system.</p>
 *
 * <p>This manager is responsible for managing the local repository of
 * packages. The method used to locate packages on local filesystem
 * and install packages is not specified.</p>
 *
 * <p>For more information about optional packages, see the document
 * <em>Optional Package Versioning</em> in the documentation bundle for your
 * Java2 Standard Edition package, in file
 * <code>guide/extensions/versioning.html</code></p>.
 *
 * @author <a href="mailto:donaldp@apache.org">Peter Donald</a>
 * @version $Revision: 1.1 $ $Date: 2001/09/03 03:21:02 $
 */
public interface PackageRepository
{
    /**
     * Return the <code>OptionalPackage</code> that provides specified
     * <code>Extension</code>. If the specified <code>Extension</code> 
     * can not be found then <code>null</code> is returned.
     *
     * @param extension Description of the optional package
     * @see #getOptionalPackages()
     * @see OptionalPackage
     * @see Extension
     */
    OptionalPackage getOptionalPackage( Extension extension );

    /**
     * Return all the <code>OptionalPackage</code>s that satisfy specified
     * <code>Extension</code>. 
     *
     * @param extension Description of the optional package
     * @see #getOptionalPackage()
     * @see OptionalPackage
     * @see Extension
     */
    OptionalPackage[] getOptionalPackages( Extension extension );

    /**
     * Install the specified File object (which represents
     * an optional package required by another package). The mechanism
     * and location of installation is not specified.
     *
     * <p>It is an error for an extension with the same
     * <code>Implementation-Version</code>, <code>Implementation-Vendor-ID</code>
     * and <code>Extension-Name</code> to already exist in Repository.</p>
     *
     * <p>It is expected that after successful completion of this
     * method that an OptionalPackage will be accessible via getOptionalPackage().</p>
     *
     * @param file the file containing "Optional Package".
     * @exception IOException if an error occurs
     * @exception SecurityException if permission has not been granted to add entries to repository
     */
    void install( final File inputFile )
        throws IOException, SecurityException;
}
