/*
 * Copyright (C) The Apache Software Foundation. All rights reserved.
 *
 * This software is published under the terms of the Apache Software License
 * version 1.1, a copy of which has been included with this distribution in
 * the LICENSE file.
 */
package org.apache.avalon.excalibur.thread;

import org.apache.avalon.framework.activity.Executable;
import org.apache.avalon.framework.logger.Loggable;
import org.apache.avalon.excalibur.pool.ObjectFactory;
import org.apache.avalon.excalibur.pool.Poolable;
import org.apache.avalon.excalibur.pool.SoftResourceLimitingPool;
import org.apache.log.Logger;

/**
 * This class is the public frontend for the thread pool code.
 *
 * TODO: Should this be configured with min threads, max threads and min spare threads ?
 *
 * @author <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 * @author <a href="mailto:donaldp@apache.org">Peter Donald</a>
 */
public class DefaultThreadPool
    extends ThreadGroup
    implements ObjectFactory, Loggable, ThreadPool
{
    private final SoftResourceLimitingPool  m_pool;
    private int                             m_level;
    private Logger                          m_logger;
    private ThreadHook                      m_hook;

    public DefaultThreadPool( final int capacity )
        throws Exception
    {
        this( "Worker Pool", capacity );
    }

    public DefaultThreadPool( final String name, final int capacity )
        throws Exception
    {
        this( name, capacity, null );
    }

    public DefaultThreadPool( final String name, 
                              final int capacity, 
                              final ThreadHook hook )
        throws Exception
    {
        super( name );
        m_pool = new SoftResourceLimitingPool( this, capacity );
        m_hook = hook;
    }

    public void setLogger( final Logger logger )
    {
        m_logger = logger;
        m_pool.setLogger( m_logger );

        try
        {
            m_pool.initialize();
        }
        catch( final Exception e )
        {
            m_logger.fatalError( "Could not initialize the pool", e );
        }
    }

    public Object newInstance()
    {
        final String name = getName() + " Worker #" + m_level++;
        final WorkerThread worker = new WorkerThread( this, name, m_pool, m_hook );
        worker.setLogger( m_logger );
        worker.start();
        return worker;
    }

    public void decommission( final Object object )
    {
        if( object instanceof WorkerThread )
        {
            ((WorkerThread)object).dispose();
        }
    }

    public Class getCreatedClass()
    {
        return WorkerThread.class;
    }

    /**
     * Run work in separate thread.
     * Return a valid ThreadControl to control work thread.
     *
     * @param work the work to be executed.
     * @return the ThreadControl
     */
    public ThreadControl execute( final Runnable work )
    {
        return execute( new ExecutableRunnable( work ) );
    }

    /**
     * Run work in separate thread.
     * Return a valid ThreadControl to control work thread.
     *
     * @param work the work to be executed.
     * @return the ThreadControl
     */
    public ThreadControl execute( final Executable work )
    {
        final WorkerThread worker = getWorker();
        return worker.execute( work );
    }

    /**
     * Run work in separate thread.
     * Wait till work is complete before returning.
     *
     * @param work the work to be executed.
     * @exception ThreadException if the work throws an exception
     */
    public void executeAndWait( final Runnable work ) 
        throws ThreadException
    {
        executeAndWait( new ExecutableRunnable( work ) );
    }

    /**
     * Run work in separate thread.
     * Wait till work is complete before returning.
     *
     * @param work the work to be executed.
     * @exception ThreadException if the work throws an exception
     */
    public void executeAndWait( final Executable work ) 
        throws ThreadException
    {
        final WorkerThread worker = getWorker();
        worker.executeAndWait( work );
    }

    /**
     * Retrieve a worker thread from pool.
     *
     * @return the worker thread retrieved from pool
     * @exception Exception if an error occurs
     */
    protected WorkerThread getWorker()
    {
        try
        {
            final WorkerThread worker = (WorkerThread)m_pool.get();
            worker.setContextClassLoader( Thread.currentThread().getContextClassLoader() );
            return worker;
        }
        catch( final Exception e )
        {
            throw new IllegalStateException( "Unable to access thread pool due to " + e );
        }
    }
}
